<?php

declare (strict_types=1);
namespace ScopedGtbabel\Gettext\Scanner;

use ScopedGtbabel\Gettext\Translation;
/**
 * Trait with common gettext function handlers
 */
trait FunctionsHandlersTrait
{
    protected function gettext(\ScopedGtbabel\Gettext\Scanner\ParsedFunction $function) : ?\ScopedGtbabel\Gettext\Translation
    {
        if (!$this->checkFunction($function, 1)) {
            return null;
        }
        list($original) = $function->getArguments();
        return $this->addComments($function, $this->saveTranslation(null, null, $original));
    }
    protected function ngettext(\ScopedGtbabel\Gettext\Scanner\ParsedFunction $function) : ?\ScopedGtbabel\Gettext\Translation
    {
        if (!$this->checkFunction($function, 2)) {
            return null;
        }
        list($original, $plural) = $function->getArguments();
        return $this->addComments($function, $this->saveTranslation(null, null, $original, $plural));
    }
    protected function pgettext(\ScopedGtbabel\Gettext\Scanner\ParsedFunction $function) : ?\ScopedGtbabel\Gettext\Translation
    {
        if (!$this->checkFunction($function, 2)) {
            return null;
        }
        list($context, $original) = $function->getArguments();
        return $this->addComments($function, $this->saveTranslation(null, $context, $original));
    }
    protected function dgettext(\ScopedGtbabel\Gettext\Scanner\ParsedFunction $function) : ?\ScopedGtbabel\Gettext\Translation
    {
        if (!$this->checkFunction($function, 2)) {
            return null;
        }
        list($domain, $original) = $function->getArguments();
        return $this->addComments($function, $this->saveTranslation($domain, null, $original));
    }
    protected function dpgettext(\ScopedGtbabel\Gettext\Scanner\ParsedFunction $function) : ?\ScopedGtbabel\Gettext\Translation
    {
        if (!$this->checkFunction($function, 3)) {
            return null;
        }
        list($domain, $context, $original) = $function->getArguments();
        return $this->addComments($function, $this->saveTranslation($domain, $context, $original));
    }
    protected function npgettext(\ScopedGtbabel\Gettext\Scanner\ParsedFunction $function) : ?\ScopedGtbabel\Gettext\Translation
    {
        if (!$this->checkFunction($function, 3)) {
            return null;
        }
        list($context, $original, $plural) = $function->getArguments();
        return $this->addComments($function, $this->saveTranslation(null, $context, $original, $plural));
    }
    protected function dngettext(\ScopedGtbabel\Gettext\Scanner\ParsedFunction $function) : ?\ScopedGtbabel\Gettext\Translation
    {
        if (!$this->checkFunction($function, 3)) {
            return null;
        }
        list($domain, $original, $plural) = $function->getArguments();
        return $this->addComments($function, $this->saveTranslation($domain, null, $original, $plural));
    }
    protected function dnpgettext(\ScopedGtbabel\Gettext\Scanner\ParsedFunction $function) : ?\ScopedGtbabel\Gettext\Translation
    {
        if (!$this->checkFunction($function, 4)) {
            return null;
        }
        list($domain, $context, $original, $plural) = $function->getArguments();
        return $this->addComments($function, $this->saveTranslation($domain, $context, $original, $plural));
    }
    protected abstract function addComments(\ScopedGtbabel\Gettext\Scanner\ParsedFunction $function, ?\ScopedGtbabel\Gettext\Translation $translation) : ?\ScopedGtbabel\Gettext\Translation;
    protected abstract function checkFunction(\ScopedGtbabel\Gettext\Scanner\ParsedFunction $function, int $minLength) : bool;
    protected abstract function saveTranslation(?string $domain, ?string $context, string $original, string $plural = null) : ?\ScopedGtbabel\Gettext\Translation;
}
